# SPDX-License-Identifier: MIT
#
# Based on:
#   https://github.com/packit/rpm-shim
#   Copyright Contributors to the Packit project.
# libdnf5-shim itself is:
#   Copyright (C) 2023 Maxwell G <maxwell@gtmx.me>

"""
libdnf5 shim module implmentation
"""

from __future__ import annotations

import importlib
import json
import logging
import subprocess
import sys
from collections.abc import Iterator
from pathlib import Path

PROJECT_NAME = "libdnf5-shim"
MODULE_NAME = "libdnf5"

INTERPRETERS: tuple[str, ...] = (
    "/usr/libexec/platform-python",
    f"/usr/bin/python{sys.version_info.major}",
    f"/usr/bin/python{sys.version_info.major}.{sys.version_info.minor}",
)
FAILURE_MSG = (
    "Failed to import system libdnf5 module. "
    "Make sure libdnf5 Python bindings installed on your system."
)

LOG = logging.getLogger(PROJECT_NAME)


def query_interp_sitepackages(interpreter) -> list[str]:
    command = (
        interpreter,
        "-c",
        "import json, site; print(json.dumps(site.getsitepackages()))",
    )
    output = subprocess.run(command, check=True, capture_output=True, text=True)
    return json.loads(output.stdout)


def get_system_sitepackages() -> Iterator[str]:
    """
    Gets a list of sitepackages directories of system Python interpreter(s).

    Returns:
        An iterator of paths.
    """

    for interpreter in INTERPRETERS:
        if not Path(interpreter).is_file():
            continue
        sitepackages = query_interp_sitepackages(interpreter)
        LOG.debug("Collected sitepackages for %s:\n%s", interpreter, sitepackages)
        yield from sitepackages


def try_path(path: str) -> bool:
    """
    Tries to load system libdnf5 module from the specified path.

    Returns:
        True if successful, False otherwise.
    """
    if not (Path(path) / MODULE_NAME).is_dir():
        return False
    sys.path.insert(0, path)
    try:
        importlib.reload(sys.modules[MODULE_NAME])
    except Exception as exc:  # pragma: no cover
        LOG.debug("Exception:", exc_info=exc)
        return False
    else:
        return True
    finally:
        del sys.path[0]


def initialize() -> None:
    """
    Initializes the shim. Tries to load system libdnf5 module.
    """
    for path in get_system_sitepackages():
        LOG.debug(f"Trying {path}")
        if try_path(path):
            LOG.debug("Import successful")
            return
    raise ImportError(FAILURE_MSG)



# Copyright (C) 2023 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: MIT

from __future__ import annotations

import sys
from types import ModuleType

import pytest
from pytest_mock import MockerFixture


def basic(mod: ModuleType):
    assert mod.__name__ == "libdnf5"
    base = mod.base.Base()
    base.load_config_from_file()
    base.get_config()
    base.setup()


def test_libdnf5_import(mocker: MockerFixture):
    import _libdnf5_shim._impl

    initializes = mocker.spy(_libdnf5_shim._impl, "initialize")

    # Import libdnf5 for the first time
    import libdnf5

    # Check that the shim was used
    initializes.assert_called_once()
    assert "libdnf5" in sys.modules
    assert "_libdnf5_shim.initialize" in sys.modules

    # Run basic tests to make sure the imported libdnf5 works
    basic(libdnf5)

    # Import again
    import libdnf5

    basic(libdnf5)

    # Ensure that initialize was only called once
    initializes.assert_called_once()


def test_libdnf5_import_fail(mocker: MockerFixture):
    import _libdnf5_shim._impl

    initializes = mocker.spy(_libdnf5_shim._impl, "initialize")

    # Set the INTERPRETERS to an empty tuple to cause a failure
    inters = _libdnf5_shim._impl.INTERPRETERS
    _libdnf5_shim._impl.INTERPRETERS = ()

    # Check failure
    try:
        with pytest.raises(ImportError, match=_libdnf5_shim._impl.FAILURE_MSG):
            import libdnf5
    finally:
        _libdnf5_shim._impl.INTERPRETERS = inters

    # Check that shim was used
    initializes.assert_called_once()

    # Try to import libdnf5
    import libdnf5  # noqa: F811

    # Check that it works now
    basic(libdnf5)
    assert initializes.call_count == 2



# Copyright (C) 2023 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: MIT

from __future__ import annotations

# Import this module for its side effects
import _libdnf5_shim.initialize  # noqa: F401



# Copyright (C) 2023 Maxwell G <maxwell@gtmx.me>
# SPDX-License-Identifier: MIT

"""
Import this module to initialize the libdnf5 shim
"""

from __future__ import annotations

from ._impl import initialize

initialize()
